; undoc.asm -- test as6502 assembler with undocumented instructions
;              may be reproduced freely

; July 1, 1998 -- first version by John Saeger (john@whimsey.com)
;                 Please report any errors!

; July 7, 1998 -- changed undocumented NOP's from NOP to NOOP
;		  added USBC #1 (eb)
;		  added JAM (02)
;		  fixed some typo's

; mnemonics are from 6510core.c by Ettore Perazzoli (ettore@comm2000.it)
; which is part of the VICE Commodore 64 emulator

; description of ARR opcode in decimal mode taken verbatim
; from 64doc.txt by John West    (john@ucc.gu.uwa.edu.au)
;               and Marko Makela (Marko.Makela@HUT.FI)

; please see 64doc.txt for more details on undocumented instruction behavior
; (available on many FTP sites all over the net)


;	processor 6502	; needed for DASM

			; op clk description (clocks with * mean page change causes 
			;                     additional clock)

	anc	#1	; 0b  2  AND accumulator with operand,
			;        set N and Z flags from result, 
			;        set Carry from bit 7 of result (high order bit).

	ane	#1	; 8b  2  OR the accumulator with the value 0eeh,
			;        AND the accumulator with the X register,
			;        AND the accumulator with the operand,
			;        set N and Z flags from the result.
			;
			;        This opcode (8b) has been reported to be unreliable on the 
			;        Commodore 64, with problems having to do with interrupts.

	arr	#1	; 6b  2  binary mode:
			;        AND accumulator with operand,
			;        shift accumulator right one bit including Carry into bit 7,
			;        set N and Z flags from the result,
			;        set C from bit 6 of the result,
			;        set V from XOR of bit 5 and bit 6 of result.
			;
			;        decimal mode:
			;        "In Decimal mode (D flag set), the ARR instruction first
			;         performs the AND and ROR, just like in Binary mode.  The
			;         N flag will be copied from the initial C flag, and the
			;         Z flag will be set according to the ROR result, as expected.
			;         The V flag well be set if the bit 6 of the accumulator
			;         changed its state between the AND and the ROR, cleared
			;         otherwise."
			;
			;        "Now comes the funny part.  If the low nybble of the AND
			;         result, incremented by its lowmost bit, is greater than 5,
			;         the low nybble in the ROR result will be incremented by 6.
			;         The low nybble may overflow as a consequence of this BCD
			;         fixup, but the high nybble won't be adjusted.  The high
			;         nybble will be BCD fixed in a similar way.  If the high
			;         nybble of the AND result, incremented by its lowmost bit,
			;         is greater than 5, the high nybble in the ROR result will
			;         be incremented by 6, and the Carry flag will be set.
			;         Otherwise the C flag will be cleared."

	asr	#1	; 4b  2  AND accumulator with operand,
			;        shift accumulator right one bit, including bit 0 into Carry,
			;        set N and Z flags from result.

	dcp	1	; c7  5  DEC operand and CMP with accumulator,
	dcp	1,x	; d7  6  flags set according to CMP logic.
	dcp	1000	; cf  6
	dcp	1000,x	; df  7
	dcp	1000,y	; db  7
	dcp	(1,x)	; c3  8
	dcp	(1),y	; d3  8

	isb	1	; e7  5  INC operand and SBC from accumulator,
	isb	1,x	; f7  6  flags set according to SBC logic.
	isb	1000	; ef  6
	isb	1000,x	; ff  7
	isb	1000,y	; fb  7
	isb	(1,x)	; e3  8
	isb	(1),y	; f3  8

	jam		; 02     Lock machine. (Tilt!)
			;        other jams are 12, 22, 32, 42, 52, 62, 72, 92, b2, d2, f2

	las	1000,y	; bb  4* AND SP register with operand,
			;        copy result to accumulator, X register, and SP register.

	lax	1	; a7  3  Load accumulator and X register with operand,
	lax	1,y	; b7  4  Set N and Z from result.
	lax	1000	; af  4
	lax	1000,y	; bf  4*
	lax	(1,x)	; a3  6
	lax	(1),y	; b3  5*

	lxa	#1	; ab  2  OR accumulator with the value 0eeh,
			;        AND accumulator with operand,
			;        copy result to X register,
			;        set N and Z from result.
			;
			;        This opcode (ab) has been reported to be unreliable on the 
			;        Commodore 64, with problems having to do with interrupts.

	nop		; ea  2  original nop should still work

	noop	#1	; 80  2  there are lots more of these, we do one of each type
	noop	1	; 04  3  (opcodes 82, c2, and e2 are normally nop but sometimes 
	noop	1,x	; 14  4   jam on C64)
	noop	1000	; 0c  4
	noop	1000,x	; 1c  4*

	rla	1	; 27  5  ROL the operand,
	rla	1,x	; 37  6  AND the accumulator with the result,
	rla	1000	; 2f  6  set N and Z from the new result.
	rla	1000,x	; 3f  7
	rla	1000,y	; 3b  7
	rla	(1,x)	; 23  8
	rla	(1),y	; 33  8

	rra	1	; 67  5  ROR the operand,
	rra	1,x	; 77  6  ADC the result to the accumulator,
	rra	1000	; 6f  6  flags set according to ADC logic.
	rra	1000,x	; 7f  7
	rra	1000,y	; 7b  7
	rra	(1,x)	; 63  8
	rra	(1),y	; 73  8

	sax	1	; 87  3  Store (accumulator AND X register).
	sax	1,y	; 97  4
	sax	1000	; 8f  4
	sax	(1,x)	; 83  6

	sbx	#1	; cb  2  AND X register with accumulator,
			;        Subtract operand from X register,
			;        set N, Z, and C from result (using CMP logic).

	sha	1000,y	; 9f  5  Store ((hi address byte + 1) AND accumulator AND X register).
	sha	(1),y	; 93  6

	shs	1000,y	; 9b  5  Store ((hi address byte + 1) AND accumulator AND X register),
			;        SP register = (accumulator AND X register).

	shx	1000,y	; 9e  5  Store ((hi address byte + 1) AND X register).

	shy	1000,x	; 9c  5  Store ((hi address byte + 1) AND Y register).

	slo	1	; 07  5  Shift operand left one bit, including bit 7 into Carry,
	slo	1,x	; 17  6  OR accumulator with the result,
	slo	1000	; 0f  6  set N and Z flags from the new result.
	slo	1000,x	; 1f  7
	slo	1000,y	; 1b  7
	slo	(1,x)	; 03  8
	slo	(1),y	; 13  8

	sre	1	; 47  5  Shift operand right one bit, including bit 0 into Carry.
	sre	1,x	; 57  6  XOR accumulator with the result,
	sre	1000	; 4f  6  set N and Z flags from the new result.
	sre	1000,x	; 5f  7
	sre	1000,y	; 5b  7
	sre	(1,x)	; 43  8
	sre	(1),y	; 53  8

	usbc	#1	; eb  2  identical to sbc #1
